{****************************************************************************}
{ RSS Fetcher Rotate v1.1 Plugin for Samurize                                }
{ -------------------------------------------                                }
{                                                                            }
{ The whole idea for this plugin came to me when i was                       }
{ looking through the Delphi code examples for Samurize.                     }
{ After checking out the code for the rss_reader plugin                      }
{ which i have been using for some time, i found out that                    }
{ it fetches the RSS feed and writes a temp file and even                    }
{ reading this tempfile, according to your project's update                  }
{ interval! this is pretty bad, when your update interval                    }
{ is set to something like 500ms (like mine). This would                     }
{ cause a heavy load on the server, hosting the RSS feed.                    }
{ So i decided to write my own rss_reader, with it's own                     }
{ Update Interval and it dosen't download any temp files.                    }
{ All string handling is done by a Stream, thus leaving                      }
{ all data in memory.                                                        }
{                                                                            }
{ The installation is simple, like any other plugin for Samurize:            }
{                                                                            }
{ 1) Copy the rss_fetcher.dll to your plugins dir                            }
{ 2) In the editor, right click and select "Add Plugin"                      }
{ 3) Select "rss_fetcher.dll" in the dropdown menu in the bottom             }
{ 4) The function is called "Fetch_RSS"                                      }
{ 5) It will ask for some parameters, which should be pretty easy            }
{    to figure out ;)                                                        }
{ 6) Done                                                                    }
{                                                                            }
{                                                                            }
{ Problems and/or comments, can be sent to zipo@it.dk                        }
{                                                                            }
{ Wed Mar 11 2004 1.1 Jens <j.theess@tu-bs.de>                               }
{ - Return the link to the currently displayed news headline.                }
{ - Fixed a problem if the rss feed can not be read.                         }
{                                                                            }
{ Wed Mar 11 2004 1.0 Jens <j.theess@tu-bs.de>                               }
{ - Modified the RSS Fetcher v1.0 Plugin to rotate between RSS items.        }
{ - Fixed a potential memory leak.                                           }
{ - Added flags to not use the IE cache.                                     }
{                                                                            }
{****************************************************************************}

library rss_fetcher;

uses
  Windows,
  SysUtils,
  Wininet, //Used to the download of the file, remove if not used , to reduce memory usage
  Classes; //For the TStringStream

var
  LastCall: TDateTime;
  LastRotate: TDateTime;
  ItemIndex: Integer;
  StrStream: TStringStream;
  TempString: string;
  Final: string;
  FirstTime: Boolean;

const
  OneDay = 1.0;
  OneHour = OneDay / 24.0;
  OneMinute = OneHour / 60.0;
  OneSecond = OneMinute / 60.0;
  OneMillisecond = OneSecond / 1000.0;

// DllMain
procedure DllMain(reason: integer);
begin
  case reason of
    DLL_PROCESS_ATTACH: // DLL loading...
    begin
      StrStream := TStringStream.Create(TempString);
      LastCall := Now;
      LastRotate := Now;
      ItemIndex := 1;
      FirstTime := True;
    end;
    DLL_PROCESS_DETACH: // DLL unloading...
    begin
      StrStream.Free; // Clean up
    end;
  end;
end;

// Function to add xx minutes to a TDateTime
function AddMinutes(const D: TDateTime; const N: Integer): TDateTime;
begin
  Result := D + OneMinute * N;
end;

// Function to add xx seconds to a TDateTime
function AddSeconds(const D: TDateTime; const N: Integer): TDateTime;
begin
  Result := D + OneSecond * N;
end;

// Function to get a file from the net and write it to a TStringStream
function GetInetFile(const FileURL: string; TempStream: TStringStream): boolean;
const
  BufferSize = 1024;
var
  hSession, hURL: HInternet;
  Buffer: array[1..BufferSize] of Byte;
  BufferLen: DWORD;
  sAppName: string;
begin
  Result := False;
  sAppName := ExtractFileName(ParamStr(0)); // By using this instead of Application.ExeName, we reduce the filesize by ~300kb !
  hSession := InternetOpen(PChar(sAppName), INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  try
    hURL := InternetOpenURL(hSession, PChar(FileURL), nil, 0, INTERNET_FLAG_PRAGMA_NOCACHE or INTERNET_FLAG_RELOAD or INTERNET_FLAG_HYPERLINK or INTERNET_FLAG_NO_CACHE_WRITE, 0);
    try
      TempStream.Size := 0; // Clear the StringStream...can maybe be done more nicely ?
      repeat
        InternetReadFile(hURL, @Buffer, SizeOf(Buffer), BufferLen);
        TempStream.Write(Buffer, BufferLen);
      until BufferLen = 0;
      Result := True;
    finally
      InternetCloseHandle(hURL)
    end
  finally
    InternetCloseHandle(hSession)
  end
end;

// Function to return string between two strings boundaries (get something inside a tag for exemple)
function GetText(Start: string; Stop: string; Source: string): string;
var
  N: Integer;
begin
  N := Pos(Start, Source);
  Source := Copy(Source, N + Length(Start), Length(Source));
  N := Pos(Stop, Source);
  GetText := Copy(Source, 1, N - 1);
end;

// Function to return a string after one string boundary
function After(Str: string; Source: string): string;
var
  N: Integer;
begin
  N := Pos(Str, Source);
  After := Copy(Source, N + Length(Str), Length(Source));
end;

// Procedure that is given control over some objects to define the DLL usage and functions
function Init(): PChar; stdcall;
begin
  Result := 'FetchRSS';
end;

// Procedure to Init the parameters of a function
function GetParam(Func: PChar): PChar; stdcall;
begin
  if Func = 'FetchRSS' then
  begin
    Result := 'RSS Feed URL|number of news to display(1-10)|Update Interval (in minutes)|Rotate Interval (in seconds)|';
  end
  else
    Result := '';
end;

// The main function ;)
function FetchRSS(Url: PChar; Number: PChar; UpdateInt: PChar; RotateInt: PChar): PChar; stdcall;
var
  S, Item, Title, Link: string;
  I, C: Integer;
begin
  if FirstTime then // (functions first time run) Then get the file
  begin
    FirstTime := False;
    GetInetFile(Url, StrStream);
  end;

  if Now >= AddMinutes(LastCall, StrToInt(string(UpdateInt))) then
  begin
    LastCall := Now;
    GetInetFile(Url, StrStream);
  end;

  if Now >= AddSeconds(LastRotate, StrToInt(string(RotateInt))) then
  begin
    LastRotate := Now;
	  ItemIndex := ItemIndex + 1;
  end;

  if StrStream.size <> 0 then
  begin
    C := StrToInt(string(Number));
    S := StrStream.DataString;

    if ItemIndex > C then
    begin
  	  ItemIndex := 1;
    end;

    Final := '';

    for I := 1 to ItemIndex do
    begin
      Item := GetText('<item>', '</item>', S);
      Title := GetText('<title>', '</title>', Item);
      Link := GetText('<link>', '</link>', Item);

      S := After('</item>', S);
      if ItemIndex = I then
      	Final := Title + '%l' + Link;
    end;

    if Final = '' then
      Final := ' ';
  end;

  Result := PChar(Final);
end;

// Here the exports names must match the ones you sended to samurize
exports Init name 'init';
exports GetParam name 'getparam';
exports FetchRSS name 'FetchRSS';

// Stuff to be done when the DLL loads
begin
  DllProc := @DllMain;
  DllProc(DLL_PROCESS_ATTACH);
end.

